function [sys,x0,str,ts] = SFr2taur(t,x,u,flag,R)

% This function computes the joint torques taur due to
% the actuator forces

% Switch based upon the current flag value
switch flag,
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes;
  case 3,
    sys=mdlOutputs(t,x,u,R);
  case {1,2,4,9}
    sys=[];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);
end

% end SFr2taur
%
%======================
% mdlInitializeSizes
%======================
%
function [sys,x0,str,ts]=mdlInitializeSizes

sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 4;
sizes.NumInputs      = 13;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed

sys = simsizes(sizes);
x0  = [];
str = [];
ts  = [0 0];

% end mdlInitializeSizes
%
%========================
% mdlOutputs
%========================
%
function sys=mdlOutputs(t,x,u,R)

Fr=u(1:4);
yc=u(5:9);
q=u(10:13);

% This function computes the joint torques due to the rod forces acting on the links

% R-matrix cell contents
%        1   2   3   4   5   6   7    8     9       10      11	  12   13   14
% 1  R=[R01 R0N RAN RJM R1A R0M R0K  R0J  p11K(1) p11K(2) p11K(3) T01A T10K TMJ0 q1c;
% 2     R1C R12 R2C R1B R2B 0   0    0    0       0       0       TB12 T12C 0 	0;
% 3     R2D RDE R23 R2E R3D REF RDF  R3F  0       0       0       TD23 TDFE T23D 0;
% 4     R34 R3G R4G RFH RGH 0   0    0    0       0       0       TG34 0    0 	0;];

% Extract necessary dimensions from R matrix
R0J=R(1,8);
p11K3=R(1,11);
RQK=R(1,1)+R(1,9);
R0M=R(1,6);
RJM=R(1,4);
T10K=R(1,13);
R0K=R(1,7);
TB12=R(2,12);
R1B=R(2,4);
R1A=R(1,5);
TD23=R(3,12);
R2D=R(3,1);
R2C=R(2,3);
T23D=R(3,14);
TG34=R(4,12);
R3F=R(3,8);
R3G=R(4,2);
RFH=R(4,4);
RGH=R(4,5);
REF=R(3,6);
TDFE=R(3,13);

% ========== Link 1 (Base) ============

%TQ0J=atan(R0M/RJM)+q(1);

%RJQ=sqrt(R0J^2+p11K3^2-2*R0J*p11K3*cos(TQ0J));
%Tra=acos((RQK^2+yc(1)^2-RJQ^2)/(2*RQK*yc(1)));

%TJp0Qp=atan(R0M/RJM)-q(1);


%RJpQp=sqrt(R0J^2+p11K3^2-2*R0J*p11K3*cos(TJp0Qp));
%Trb=2*pi-acos((RQK^2+yc(2)^2-RJpQp^2)/(2*RQK*yc(2)));

%p10K=[RQK 0 R(1,11)]';
%p10Kp=[RQK 0 -R(1,11)]';

% Note assumption that swing cylinders contribute equal forces
%F1ra=Fr(1)*[cos(Tra) 0 -sin(Tra)]';  
%F1rb=-Fr(2)*[cos(Trb) 0 -sin(Trb)]'; 

%taur(1)=dot((cross(p10K,F1ra)+cross(p10Kp,F1rb)),[0 1 0]');

%**************************************************************
%                   TEMP FIX SECTION    
% ***Approximate moment arm of swing cylinders as a constant***
taur(1)=2*Fr(1)*RJM;
% see also "Sjoint_to_cyl", "cylinders"
%**************************************************************

% ========== Link 2 (Boom) ============

T1BA=acos((yc(3)^2+R1B^2-R1A^2)/(2*yc(3)*R1B));
Tr2=2*pi+TB12-T1BA;

p21B=R1B*[cos(TB12) sin(TB12) 0]';
F2r2=Fr(2)*[cos(Tr2) sin(Tr2) 0]';
taur(2)=dot(cross(p21B,F2r2),[0 0 1]');

% ========== Link 3 (Stick) ============

T2DC=acos((R2D^2+yc(4)^2-R2C^2)/(2*R2D*yc(4)));
Tx3p2D=pi-TD23;
Tr3=pi-T2DC-Tx3p2D;

p32D=R2D*[cos(TD23) sin(TD23) 0]';
F3r3=Fr(3)*[cos(Tr3) sin(Tr3) 0]';
taur(3)=dot(cross(p32D,F3r3),[0 0 1]');

% ========== Link 4 (Bucket) ============

Tx33G=2*pi-q(4)-TG34;
if q(4)+TG34>2*pi
   TG3F=pi-Tx33G-T23D;
else
   TG3F=pi-Tx33G+T23D;
end
RFG=sqrt(R3F^2+R3G^2-2*R3F*R3G*cos(TG3F));
TFG3=acos((RFG^2+R3G^2-R3F^2)/(2*RFG*R3G));
TFGH=acos((RFG^2+RGH^2-RFH^2)/(2*RFG*RGH));
if q(4)+TG34>2*pi
   T3GH=TFGH+TFG3;
else
   T3GH=TFGH-TFG3;
end
TG=TG34-T3GH;
T3FG=acos((R3F^2+RFG^2-R3G^2)/(2*R3F*RFG));
THFG=acos((RFG^2+RFH^2-RGH^2)/(2*RFG*RFH));
if q(4)+TG34>2*pi
   THF3=THFG+T3FG;
else
   THF3=THFG-T3FG;
end
T3FHp=pi-THF3;
TF=(pi-T3FHp-T23D)+pi;

TEFH=pi-TDFE-THF3;
yc(5)=sqrt(REF^2+RFH^2-2*REF*RFH*cos(TEFH));
TFHE=acos((RFH^2+yc(5)^2-REF^2)/(2*RFH*yc(5)));
Tr4=-T23D+THF3-TFHE;

A=[cos(TF) cos(q(4)+TG-pi);
   sin(TF) sin(q(4)+TG-pi);];
b=[-Fr(4)*cos(Tr4);-Fr(4)*sin(Tr4)];
x=inv(A)*b;
FG=x(2);

p43G=R3G*[cos(TG34) sin(TG34) 0]';
F4G=FG*[cos(TG) sin(TG) 0]';
taur(4)=dot(cross(p43G,F4G),[0 0 1]');

sys = taur;

% end mdlOutputs
