%===================================================================================
%
%						TRENCHING TRAJECTORY DEMONSTRATION
%
% This function computes an Nx5 matrix representing bucket displacement
% throughout the digging motion
%
% traj=[t(1) x(1) y(1) z(1) phi(1);
%		 t(2) x(2) y(2) z(2) phi(2);
%        .    .    .     .    .   ; 
%        .    .    .     .    .   ; 
%        .    .    .     .    .   ; 
%		 t(N) x(N) y(N) z(N) phi(N); 
%
% Where t is time, x,y,z are the position of the bucket tip relative to the
% base frame in frame 0 coordinates, and phi is the angle of the bucket wrt 
% the horizontal x0-y0 plane, positive downward
% 
%===================================================================================

clear all ; close all ; clc ;

% Set up the initial configuration and initialize the output matrix  
tfinal=60;
N=3000;
home=[20 0 2 -10*pi/180];
start=home;
traj=[[0 home];zeros(N-1,5)];
a=[8.5000   48.0000   38+11/16   18.5000];
qmax =[1.5708    1.0907    6.1403    6.5025];
qmin =[-1.5708   -0.8529    3.6469    4.1089];
   
% Define trajectory dimensions
zg=3;
reach=52;
Tshear=30;
plunge=3;
rA=4;
scoop=40;
rB=4;
swingH=1;
Tlift=10;
Tdump=130;
Xdump=50;
Ydump=12;
dumpH=22;
Bang=20;

% Define the time intervals
% (time intervals are normalized and only relative to the others)
interval=[10 5 5 2 10 5 10 5 5 5 5 5 1]';
tot=sum(interval);
fraction=interval/tot;
ti=zeros(13,1);
for i=1:13
   for j=1:i
      ti(i)=ti(i)+fraction(j)*tfinal;
   end   
end

% Set interval endpoints
t0=0;    		% home position
t1=ti(1);		% move to start position
t2=ti(2);		% reach
t3=ti(3);		% plunge
t4=ti(4);		% curlA
t5=ti(5);		% scoop
t6=ti(6);		% curlB
t7=ti(7);		% lift
t8=ti(8);		% swing out
t9=ti(9);		% dump
t10=ti(10);		% retract
t11=ti(11);		% swing back
t12=ti(12);		% return to start
t13=ti(13);		% return to home

dt=tfinal/N;

% =========== Set up positions at interval endpoints ===============
% home position
x0=home(1);
y0=home(2);
z0=home(3);
phi0=home(4);

% start position
x1=start(1);
y1=start(2);
z1=start(3);
phi1=start(4);

% reached out position
x2=x1+reach;
y2=0;
z2=-zg;
phi2=pi/2+Bang*pi/180;

% plunged position
x3=x2;
y3=0;
z3=z2-plunge;
phi3=phi2;

% curled A position
x4=x3-rA;
y4=0;
z4=z3-rA;
phi4=(Bang+Tshear)*pi/180;

% scooped position
x5=x4-scoop;
y5=0;
z5=z4;
phi5=phi4;

% curled B position
x6=x5-rB;
y6=0;
z6=z5+rB;
phi6=Tlift*pi/180;

% lifted position
x7=x6;
y7=0;
z7=swingH-zg;
phi7=phi6;

% swung out position
alphaSWf=atan(Ydump/Xdump);
x8=Xdump-a(4)*cos(alphaSWf)*cos(Tlift*pi/180);
y8=Ydump-a(4)*sin(alphaSWf)*cos(Tlift*pi/180);
z8=dumpH+a(4)*sin(Tlift*pi/180)-zg;
phi8=phi7;

% dumped position
x9=Xdump+a(4)*cos(alphaSWf)*cos(pi-Tdump*pi/180);
y9=Ydump+a(4)*sin(alphaSWf)*cos(pi-Tdump*pi/180);
z9=dumpH-a(4)*sin(pi-Tdump*pi/180)-zg;
phi9=Tdump*pi/180;

% retracted position
x10=x8;
y10=y8;
z10=z8;
phi10=phi8;

% swung back position
x11=x7;
y11=y7;
z11=z7;
phi11=phi7;

% returned start position
x12=x1;
y12=y1;
z12=z1;
phi12=phi1;

% returned to home position
x13=x0;
y13=y0;
z13=z0;
phi13=phi0;

% ============ Loop through the time steps ================
t(1)=0;
alphaSW(1)=0;
for i=2:N
   t(i)=t(i-1)+dt;
   alphaSW(i)=0;
   if t(i) < t1										% t0->t1: moving to start					
      flag=' moving to start';
      x(i)=linear(x0,x1,t0,t1,t(i));
      y(i)=linear(y0,y1,t0,t1,t(i));
      z(i)=linear(z0,z1,t0,t1,t(i));
      phi(i)=linear(phi0,phi1,t0,t1,t(i));
      
   elseif t1 < t(i) & t(i) < t2					% t1->t2: reaching out
      flag=' reaching out';
      x(i)=linear(x1,x2,t1,t2,t(i));
      y(i)=0;
      z(i)=linear(z1,z2,t1,t2,t(i));
      phi(i)=linear(phi1,phi2,t1,t2,t(i));
      
   elseif t2 < t(i) & t(i) < t3					% t2->t3: plunging
      flag=' plunge';
      x(i)=x2;
      y(i)=0;
      z(i)=linear(z2,z3,t2,t3,t(i));
      phi(i)=phi2;
      
   elseif t3 < t(i) & t(i) < t4					% t3->t4: curl A
      flag=' curlA';
      alphaA=linear(0,pi/2,t3,t4,t(i));
      x(i)=x3-rA*(1-cos(alphaA));
      y(i)=0;
      z(i)=z3-rA*sin(alphaA);
      phi(i)=linear(phi3,phi4,t3,t4,t(i));      
      
   elseif t4 < t(i) & t(i) < t5	 				% t4->t5: scooping
      flag=' scoop';
      x(i)=linear(x4,x5,t4,t5,t(i));
      y(i)=0;
      z(i)=z4;
      phi(i)=phi4;    
      
   elseif t5 < t(i) & t(i) < t6	 				% t5->t6: curl B
      flag=' curlB';
      alphaB=linear(0,pi/2,t5,t6,t(i));
      x(i)=x5-rB*sin(alphaB);
      y(i)=0;
      z(i)=z5+rB*(1-cos(alphaB));
      phi(i)=linear(phi5,phi6,t5,t6,t(i));      
      
   elseif t6 < t(i) & t(i) < t7	 				% t6->t7: lifting
      flag=' lifting';
      x(i)=x6;
      y(i)=0;
      z(i)=linear(z6,z7,t6,t7,t(i));
      phi(i)=phi6;       
      
   elseif t7 < t(i) & t(i) < t8	 				% t7->t8: swinging out
      flag=' swing out';
      x(i)=linear(x7,x8,t7,t8,t(i));
      y(i)=linear(y7,y8,t7,t8,t(i));
      z(i)=linear(z7,z8,t7,t8,t(i));
      phi(i)=linear(phi7,phi8,t7,t8,t(i));
      alphaSW(i)=atan(y(i)/x(i));
      
   elseif t8 < t(i) & t(i) < t9	 				% t8->t9: dumping
      flag=' dump';
      phi(i)=linear(phi8,phi9,t8,t9,t(i));      
      x(i)=Xdump-a(4)*cos(alphaSWf)*cos(phi(i));
      y(i)=Ydump-a(4)*sin(alphaSWf)*cos(phi(i));
      z(i)=dumpH-a(4)*sin(phi(i))-zg;
        
   elseif t9 < t(i) & t(i) < t10		 			% t9->t10: retracting
      flag=' retract';
      phi(i)=linear(phi9,phi10,t9,t10,t(i));      
      x(i)=Xdump-a(4)*cos(alphaSWf)*cos(phi(i));
      y(i)=Ydump-a(4)*sin(alphaSWf)*cos(phi(i));
      z(i)=dumpH-a(4)*sin(phi(i))-zg;    
      
   elseif t10 < t(i) & t(i) < t11	 			% t10->t11: swinging back 
      flag=' swing back';
      x(i)=linear(x10,x11,t10,t11,t(i));
      y(i)=linear(y10,y11,t10,t11,t(i));
      z(i)=linear(z10,z11,t10,t11,t(i));
      phi(i)=linear(phi10,phi11,t10,t11,t(i));    
      
   elseif t11 < t(i) & t(i) < t12	 			% t11->t12: return to start
      flag=' return to start';
      x(i)=linear(x11,x12,t11,t12,t(i));
      y(i)=linear(y11,y12,t11,t12,t(i));
      z(i)=linear(z11,z12,t11,t12,t(i));
      phi(i)=linear(phi11,phi12,t11,t12,t(i));        
      
   elseif t(i) < t13									% t12->t13: return to home         
      flag=' return to home';
      x(i)=linear(x12,x13,t12,t13,t(i));
      y(i)=linear(y12,y13,t12,t13,t(i));
      z(i)=linear(z12,z13,t12,t13,t(i));
      phi(i)=linear(phi12,phi13,t12,t13,t(i));      
      
   else
      disp('missed a time step--trajectory error')
      
   end
   
   % Check to make sure travel limts have not been exceeded
   q=Rdisp([x(i);y(i);z(i);phi(i)],a);
   for k=1:4
      if q(k) > qmax(k) | q(k) < qmin(k)
         disp(strcat('Warning: travel limits exceeded at t=', ...
            num2str(i*dt),' sec '))
         disp(strcat('during the ',flag,' interval in joint ', num2str(k)))
         disp(strcat('bucket angle is ',num2str(phi(i)*180/pi)))
      end
   end
   
   % place current position into output matrix
   traj(i,1)=t(i);
   traj(i,2)=x(i);
   traj(i,3)=y(i);
   traj(i,4)=z(i);
   traj(i,5)=phi(i);
   
end

% plot results to visualize trajectory
plot3(x,y,z,'r')
xlabel('extension [in]')
ylabel('swing [in]')
zlabel('elevation [in]')
axis square
hold on
for i=1:N
   xwrist=x(i)+a(4)*cos(phi(i))*cos(alphaSW(i));
   ywrist=y(i)+a(4)*cos(phi(i))*sin(alphaSW(i));
   zwrist=z(i)+a(4)*sin(phi(i));
   plot3(xwrist,ywrist,zwrist,'b')
end
axis equal
grid on

tfinal=traj(end,1);
save demo_trench traj tfinal