function [pc,pr]=staticpressure(tau,yr,Ac,Ar,R)

% This function is used in the startup routine to compute the initial 
% cylinder pressures required to hold the backhoe in equilibrium against gravity.

% The forces on the rods are first computed from the (applied) joint torques,
% then the pressures are computed based on the cylinder area. Coulomb friction
% in the walls of the cylinder are neglected and the non-pressurized side of the
% cylinder is assumed zero.

% Note sign convention: all torques are positive in the direction of their 
% corresponding z-axis, all cylinder forces are positive in compression

% R-matrix cell contents
%        1   2   3   4   5   6   7    8     9       10      11	  12   13   14
% 1  R=[R01 R0N RAN RJM R1A R0M R0K  R0J  p11K(1) p11K(2) p11K(3) T01A T10K TMJ0 q1c;
% 2     R1C R12 R2C R1B R2B 0   0    0    0       0       0       TB12 T12C 0 	0;
% 3     R2D RDE R23 R2E R3D REF RDF  R3F  0       0       0       TD23 TDFE T23D 0;
% 4     R34 R3G R4G RFH RGH 0   0    0    0       0       0       TG34 0    0 	0;];

% Extract necessary dimensions from R matrix
R0J=R(1,8);
R0K=R(1,7);
R1B=R(2,4);
R1A=R(1,5);
R2D=R(3,1);
R2C=R(2,3);
R3F=R(3,8);
R3G=R(4,2);
RFH=R(4,4);
RGH=R(4,5);
REF=R(3,6);
TDFE=R(3,13);

% ========== Link 1 (Base) ============

TJK0=acos((R0K^2+yr(1)^2-R0J^2)/(2*R0K*yr(1)));
r1p=R0K*sin(TJK0);
Fr(1)=tau(1)/(2*r1p);
% Note assumption that swing cylinders contribute equal forces

if Fr(1)>0
   pc(1)=Fr(1)/Ac(1);
   pr(1)=0;
elseif Fr(1)<0
   pc(1)=0;
   pr(1)=-Fr(1)/Ar(1);
else
   pc(1)=0;
   pr(1)=0;
end

% ========== Link 2 (Boom) ============

T1BA=acos((yr(3)^2+R1B^2-R1A^2)/(2*yr(3)*R1B));
r2p=R1B*sin(T1BA);
Fr(2)=-tau(2)/r2p;
if Fr(2)>0
   pc(2)=Fr(2)/Ac(2);
   pr(2)=0;
elseif Fr(2)<0
   pc(2)=0;
   pr(2)=-Fr(2)/Ar(2);
else
   pc(2)=0;
   pr(2)=0;
end

% ========== Link 3 (Stick) ============

TDC2=acos((yr(4)^2+R2C^2-R2D^2)/(2*yr(4)*R2C));
r3p=R2C*sin(TDC2);
Fr(3)=-tau(3)/r3p;
if Fr(3)>0
   pc(3)=Fr(3)/Ac(3);
   pr(3)=0;
elseif Fr(3)<0
   pc(3)=0;
   pr(3)=-Fr(3)/Ar(3);
else
   pc(3)=0;
   pr(3)=0;
end

% ========== Link 4 (Bucket) ============

TEFH=acos((REF^2+RFH^2-yr(5)^2)/(2*REF*RFH));
THF3=pi-TEFH-TDFE;
R3H=sqrt(R3F^2+RFH^2-2*R3F*RFH*cos(THF3));
T3GH=acos((R3G^2+RGH^2-R3H^2)/(2*R3G*RGH));
r4p=R3G*sin(T3GH);
FG=-tau(4)/r4p;
T3HF=acos((RFH^2+R3H^2-R3F^2)/(2*RFH*R3H));
TGH3=acos((R3H^2+RGH^2-R3G^2)/(2*R3H*RGH));
TGHF=TGH3+T3HF;
TFHE=acos((yr(5)^2+RFH^2-REF^2)/(2*yr(5)*RFH));
Fr(4)=(sin(TGHF)/sin(TFHE))*FG;
if Fr(4)>0
   pc(4)=Fr(4)/Ac(4);
   pr(4)=0;
elseif Fr(4)<0
   pc(4)=0;
   pr(4)=-Fr(4)/Ar(4);
else
   pc(4)=0;
   pr(4)=0;
end