% This function performs symbolic manipulation of the dynamic
% variables in the formulation of the LaGrangian dynamic model

% Output is in the form of text files that may be pasted into subsequent 
% functions for numerical computations

clear all; close all; clc;

% Declare static joint variables
syms m1 m2 m3 m4
syms I111 I112 I113 I121 I122 I123 I131 I132 I133
syms I211 I212 I213 I221 I222 I223 I231 I232 I233
syms I311 I312 I313 I321 I322 I323 I331 I332 I333
syms I411 I412 I413 I421 I422 I423 I431 I432 I433
syms p1x p1y p1z p2x p2y p2z p3x p3y p3z p4x p4y p4z
syms a1 a2 a3 a4
syms g

% Declare dynamic joint variables
syms q1 q2 q3 q4
syms qd1 qd2 qd3 qd4

% Assemble symbolic inertia matrices
% These will be used to represent the inertia matrices at the local COM's
% and expressed in the local frames
I11=[I111 I112 I113;I121 I122 I123;I131 I132 I133];
I22=[I211 I212 I213;I221 I222 I223;I231 I232 I233];
I33=[I311 I312 I313;I321 I322 I323;I331 I332 I333];
I44=[I411 I412 I413;I421 I422 I423;I431 I432 I433];

% Position vectors from joint i-1 to COM i and between joints in local frames
p1o0c1=[p1x;p1y;p1z];
p1o0o1=[a1;0;0];

p2o1c2=[p2x;p2y;p2z];
p2o1o2=[a2;0;0];

p3o2c3=[p3x;p3y;p3z];
p3o2o3=[a3;0;0];

p4o3c4=[p4x;p4y;p4z];
p4o3o4=[a4;0;0];

% Rotation matrices between component frames
Rx90=[1 0 0;0 0 -1;0 1 0];
Ry1=[cos(q1) 0 sin(q1);0 1 0;-sin(q1) 0 cos(q1)];
R01=Rx90*Ry1;

R12=[cos(q2) -sin(q2) 0;sin(q2) cos(q2) 0;0 0 1];
R02=R01*R12;

R23=[cos(q3) -sin(q3) 0;sin(q3) cos(q3) 0;0 0 1];
R03=R02*R23;

R34=[cos(q4) -sin(q4) 0;sin(q4) cos(q4) 0;0 0 1];
R04=R03*R34;

% Position vectors from origins to COMs in base frame
% necessary for Jacobian
p0o0c1=R01*p1o0c1;

p0o1c2=R02*p2o1c2;
%p0o0c2=p0o1c2+R01*p1o0o1;

p0o2c3=R03*p3o2c3;
%p0o1c3=p0o2c3+R02*p2o1o2;
%p0o0c3=p0o1c3+R01*p1o0o1;

p0o3c4=R04*p4o3c4;
%p0o2c4=p0o3c4+R03*p3o2o3;
%p0o1c4=p0o2c4+R02*p2o1o2;
%p0o0c4=p0o1c4+R01*p1o0o1;

% z-axes
z00=[0;0;1];

z11=[0;0;1];
z01=R01*z11;

z22=z11;
z02=R02*z22;

z33=z11;
z03=R03*z33;

% Jacobian matrices
Jv1=[cross(z00,p0o0c1) zeros(3,3)];
Jv2=[cross(z00,p0o0c1) cross(z01,p0o1c2) zeros(3,2)];
Jv3=[cross(z00,p0o0c1) cross(z01,p0o1c2) cross(z02,p0o2c3) zeros(3,1)];
Jv4=[cross(z00,p0o0c1) cross(z01,p0o1c2) cross(z02,p0o2c3) cross(z03,p0o3c4)];

Jw1=[z00 zeros(3,3)];
Jw2=[z00 z01 zeros(3,2)];
Jw3=[z00 z01 z02 zeros(3,1)];
Jw4=[z00 z01 z02 z03];

% Rotate the inertia matrices to the base frame
I01=simple(R01*I11*R01.');
I02=simple(R02*I22*R02.');
I03=simple(R03*I33*R03.');
I04=simple(R04*I44*R04.');

% ======================== Inertia matrix =========================

% Symbolically compute inertia matrix
M=(Jv1.'*m1*Jv1+Jw1.'*I01*Jw1)+(Jv2.'*m2*Jv2+Jw2.'*I02*Jw2)+...
   (Jv3.'*m3*Jv3+Jw3.'*I03*Jw3)+(Jv4.'*m4*Jv4+Jw4.'*I04*Jw4);

% Write matrix terms to data files, element by element
for i=1:4
   for j=1:4
      Mij=simple(M(i,j));
      Mij=char(Mij); 
      fid=fopen(strcat(cd,['\M',num2str(i),num2str(j),'.txt']),'w');
      fwrite(fid,Mij);
      fclose(fid);
   end   
end

% =================== Velocity coupling vector ====================

% Symbolically compute vector terms
q=[q1;q2;q3;q4];
qd=[qd1;qd2;qd3;qd4];
for i=1:4
   clear Vi
   Vi=class('sym');
   for j=i:4
      for k=1:4
         fid=fopen(strcat(cd,'\M',num2str(i),num2str(j),'.txt'));
         Mij=fscanf(fid,'%c');
         fclose(fid);
         
         fid=fopen(strcat(cd,'\M',num2str(j),num2str(k),'.txt'));
         Mjk=fscanf(fid,'%c');
         fclose(fid);
         
         F1=diff(Mij,q(k));
         F2=diff(Mjk,q(i));
         Vi=Vi+(F1-0.5*F2)*qd(j)*qd(k);
      end
   end
   
   V=char(simple(Vi));
   V=strrep(V,'char','');
   
   
   % Write vector terms to data files, element by element
   switch i
   case 1
      V1=V;
      fid=fopen(strcat(cd,'\V1.txt'),'w');

      fwrite(fid,V1);
      fclose(fid);
   case 2
      V2=V;
      fid=fopen(strcat(cd,'\V2.txt'),'w');
      fwrite(fid,V2);
      fclose(fid);
   case 3
      V3=V;
      fid=fopen(strcat(cd,'\V3.txt'),'w');
      fwrite(fid,V3);
      fclose(fid);
   case 4
      V4=V;
      fid=fopen(strcat(cd,'\V4.txt'),'w');
      fwrite(fid,V4);
      fclose(fid);
   end
end

% ======================== Gravitational torque vector =======================

% Symbolically compute vector terms
g=sym([0;0;-g]);
m=[m1 m2 m3 m4];

for i=1:4
   clear Gi
   Gi=class('sym');
   for j=1:4
      switch j
      case 1
         Jivj=Jv1(:,i);
      case 2
         Jivj=Jv2(:,i);
      case 3
         Jivj=Jv3(:,i);
      case 4
         Jivj=Jv4(:,i);
      end
      Gi=Gi-m(j)*g.'*Jivj;
   end
   G=char(simple(Gi));
   G=strrep(G,'char','');
   
   % Write vector terms to data files, element by element
   switch i
   case 1
      G1=G;
      fid=fopen(strcat(cd,'\G1.txt'),'w');
      fwrite(fid,G1);
      fclose(fid);
   case 2
      G2=G;
      fid=fopen(strcat(cd,'\G2.txt'),'w');
      fwrite(fid,G2);
      fclose(fid);
   case 3
      G3=G;
      fid=fopen(strcat(cd,'\G3.txt'),'w');
      fwrite(fid,G3);
      fclose(fid);
   case 4
      G4=G;
      fid=fopen(strcat(cd,'\G4.txt'),'w');
      fwrite(fid,G4);
      fclose(fid);
   end
end

% Reopen the files replace all the qi & qdi terms w/ q(i) & qd(i)
for i=1:4
   
   % Inertia matrix terms
   for j=1:4
      fid=fopen(strcat(cd,'\M',num2str(i),num2str(j),'.txt'),'r');
      Mij=fscanf(fid,'%c');
      fclose(fid);
      Mij=strrep(Mij,'q1','q(1)');
      Mij=strrep(Mij,'q2','q(2)');
      Mij=strrep(Mij,'q3','q(3)');
      Mij=strrep(Mij,'q4','q(4)');
      Mij=strrep(Mij,'qd1','qd(1)');
      Mij=strrep(Mij,'qd2','qd(2)');
      Mij=strrep(Mij,'qd3','qd(3)');
      Mij=strrep(Mij,'qd4','qd(4)');
      fid=fopen(strcat(cd,'\M',num2str(i),num2str(j),'.txt'),'w');
      fwrite(fid,Mij);
      fclose(fid);
   end
   
   % Velocity coupling vector terms
   fid=fopen(strcat(cd,'\V',num2str(i),'.txt'),'r');
   Vi=fscanf(fid,'%c');
   fclose(fid);
   Vi=strrep(Vi,'q1','q(1)');
   Vi=strrep(Vi,'q2','q(2)');
   Vi=strrep(Vi,'q3','q(3)');
   Vi=strrep(Vi,'q4','q(4)');
   Vi=strrep(Vi,'qd1','qd(1)');
   Vi=strrep(Vi,'qd2','qd(2)');
   Vi=strrep(Vi,'qd3','qd(3)');
   Vi=strrep(Vi,'qd4','qd(4)');
   fid=fopen(strcat(cd,'\V',num2str(i),'.txt'),'w');
   fwrite(fid,Vi);
   fclose(fid);   
   
   % Gravity vector terms
   fid=fopen(strcat(cd,'\G',num2str(i),'.txt'),'r');
   Gi=fscanf(fid,'%c');
   fclose(fid);
   Gi=strrep(Gi,'q1','q(1)');
   Gi=strrep(Gi,'q2','q(2)');
   Gi=strrep(Gi,'q3','q(3)');
   Gi=strrep(Gi,'q4','q(4)');
   Gi=strrep(Gi,'qd1','qd(1)');
   Gi=strrep(Gi,'qd2','qd(2)');
   Gi=strrep(Gi,'qd3','qd(3)');
   Gi=strrep(Gi,'qd4','qd(4)');
   fid=fopen(strcat(cd,'\G',num2str(i),'.txt'),'w');
   fwrite(fid,Gi);
   fclose(fid);      
   
end

disp('Symbolic matrix and vector elements save successfully to folder')
disp(cd)